@@ -42,22 +42,48 @@ G_BEGIN_DECLS
 typedef struct _GMimeMessage GMimeMessage;
 typedef struct _GMimeMessageClass GMimeMessageClass;
 
+
+/**
+ * GMimeRecipientType:
+ * @GMIME_RECIPIENT_TYPE_TO: Represents the recipients in the To: header.
+ * @GMIME_RECIPIENT_TYPE_CC: Represents the recipients in the Cc: header.
+ * @GMIME_RECIPIENT_TYPE_BCC: Represents the recipients in the Bcc: header.
+ *
+ * A message recipient type.
+ **/
+typedef enum _GMimeRecipientType {
+	GMIME_RECIPIENT_TYPE_TO,
+	GMIME_RECIPIENT_TYPE_CC,
+	GMIME_RECIPIENT_TYPE_BCC,
+} GMimeRecipientType;
+
+
+/**
+ * GMimeMessage:
+ * @parent_object: parent #GMimeObject
+ * @mime_part: toplevel MIME part
+ * @recipients: hash table of recipients using recipient header name as the hash key
+ * @message_id: Message-Id string
+ * @reply_to: Reply-To string
+ * @subject: Subject string
+ * @from: From string
+ * @date: Date value
+ * @tz_offset: timezone offset
+ *
+ * A MIME Message object.
+ **/
 struct _GMimeMessage {
 	GMimeObject parent_object;
 	
-	char *from;
+	InternetAddressList **recipients;
+	GMimeObject *mime_part;
+	char *message_id;
 	char *reply_to;
-	
-	GHashTable *recipients;
-	
 	char *subject;
+	char *from;
 	
 	time_t date;
-	int gmt_offset;     /* GMT offset */
-	
-	char *message_id;
-	
-	GMimeObject *mime_part;
+	int tz_offset;
 };
 
 struct _GMimeMessageClass {
@@ -66,31 +92,6 @@ struct _GMimeMessageClass {
 };
 
 
-/**
- * GMIME_RECIPIENT_TYPE_TO:
- *
- * Recipients in the To: header.
- **/
-#define	GMIME_RECIPIENT_TYPE_TO  "To"
-
-
-/**
- * GMIME_RECIPIENT_TYPE_CC:
- *
- * Recipients in the Cc: header.
- **/
-#define	GMIME_RECIPIENT_TYPE_CC  "Cc"
-
-
-/**
- * GMIME_RECIPIENT_TYPE_BCC:
- *
- * Recipients in the Bcc: header.
- **/
-#define	GMIME_RECIPIENT_TYPE_BCC "Bcc"
-
-
-
 GType g_mime_message_get_type (void);
 
 GMimeMessage *g_mime_message_new (gboolean pretty_headers);
@@ -101,29 +102,26 @@ const char *g_mime_message_get_sender (GMimeMessage *message);
 void g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to);
 const char *g_mime_message_get_reply_to (GMimeMessage *message);
 
-void g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *name, const char *address);
-void g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, const char *str);
-const InternetAddressList *g_mime_message_get_recipients (GMimeMessage *message, const char *type);
+void g_mime_message_add_recipient (GMimeMessage *message, GMimeRecipientType type, const char *name, const char *addr);
+InternetAddressList *g_mime_message_get_recipients (GMimeMessage *message, GMimeRecipientType type);
 InternetAddressList *g_mime_message_get_all_recipients (GMimeMessage *message);
 
 void g_mime_message_set_subject (GMimeMessage *message, const char *subject);
 const char *g_mime_message_get_subject (GMimeMessage *message);
 
-void g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset);
-void g_mime_message_get_date (GMimeMessage *message, time_t *date, int *gmt_offset);
-char *g_mime_message_get_date_string (GMimeMessage *message);
+void g_mime_message_set_date (GMimeMessage *message, time_t date, int tz_offset);
+void g_mime_message_get_date (GMimeMessage *message, time_t *date, int *tz_offset);
+void g_mime_message_set_date_as_string (GMimeMessage *message, const char *str);
+char *g_mime_message_get_date_as_string (GMimeMessage *message);
 
 void g_mime_message_set_message_id (GMimeMessage *message, const char *message_id);
 const char *g_mime_message_get_message_id (GMimeMessage *message);
 
-void g_mime_message_add_header (GMimeMessage *message, const char *header, const char *value);
-void g_mime_message_set_header (GMimeMessage *message, const char *header, const char *value);
-const char *g_mime_message_get_header (GMimeMessage *message, const char *header);
-
 GMimeObject *g_mime_message_get_mime_part (GMimeMessage *message);
 void g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part);
 
-void g_mime_message_foreach_part (GMimeMessage *message, GMimePartFunc callback, gpointer data);
+void g_mime_message_foreach (GMimeMessage *message, GMimeObjectForeachFunc callback,
+			     gpointer user_data);
 
 G_END_DECLS
 
