@@ -42,11 +42,16 @@ enum {
 	GMIME_MULTIPART_SIGNED_SIGNATURE
 };
 
+
+/**
+ * GMimeMultipartSigned:
+ * @parent_object: parent #GMimeMultipart
+ *
+ * A multipart/signed MIME part.
+ **/
 struct _GMimeMultipartSigned {
 	GMimeMultipart parent_object;
 	
-	char *protocol;
-	char *micalg;
 };
 
 struct _GMimeMultipartSignedClass {
