@@ -102,7 +102,8 @@ g_mime_object_get_type (void)
 			(GInstanceInitFunc) g_mime_object_init,
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GMimeObject", &info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "GMimeObject",
+					       &info, G_TYPE_FLAG_ABSTRACT);
 	}
 	
 	return type;
@@ -344,7 +345,7 @@ g_mime_object_new (GMimeContentType *content_type)
 			return NULL;
 	}
 	
-	object = g_object_new (obj_type, NULL);
+	object = g_object_newv (obj_type, 0, NULL);
 	
 	g_mime_object_set_content_type (object, content_type);
 	
@@ -372,7 +373,6 @@ g_mime_object_new_type (const char *type, const char *subtype)
 {
 	struct _type_bucket *bucket;
 	struct _subtype_bucket *sub;
-	GMimeObject *object;
 	GType obj_type;
 	
 	g_return_val_if_fail (type != NULL, NULL);
@@ -400,9 +400,7 @@ g_mime_object_new_type (const char *type, const char *subtype)
 			return NULL;
 	}
 	
-	object = g_object_new (obj_type, NULL);
-	
-	return object;
+	return g_object_newv (obj_type, 0, NULL);
 }
 
 
