@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -26,6 +27,21 @@
 
 #include "gmime-stream-null.h"
 
+
+/**
+ * SECTION: gmime-stream-null
+ * @title: GMimeStreamNull
+ * @short_description: A null stream
+ * @see_also: #GMimeStream
+ *
+ * A #GMimeStream which has no real backing storage at all. This
+ * stream is useful for dry-runs and can also be useful for
+ * determining statistics on source data which can be written to
+ * streams but cannot be read as a stream itself (e.g. a #GMimeObject
+ * via g_mime_object_write_to_stream()).
+ **/
+
+
 static void g_mime_stream_null_class_init (GMimeStreamNullClass *klass);
 static void g_mime_stream_null_init (GMimeStreamNull *stream, GMimeStreamNullClass *klass);
 static void g_mime_stream_null_finalize (GObject *object);
@@ -36,10 +52,10 @@ static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
-static off_t stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
-static off_t stream_tell (GMimeStream *stream);
+static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
+static gint64 stream_tell (GMimeStream *stream);
 static ssize_t stream_length (GMimeStream *stream);
-static GMimeStream *stream_substream (GMimeStream *stream, off_t start, off_t end);
+static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
 static GMimeStreamClass *parent_class = NULL;
@@ -59,7 +75,7 @@ g_mime_stream_null_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamNull),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_null_init,
 		};
 		
@@ -164,11 +180,11 @@ stream_reset (GMimeStream *stream)
 	return 0;
 }
 
-static off_t
-stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
+static gint64
+stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 {
 	GMimeStreamNull *null = (GMimeStreamNull *) stream;
-	off_t bound_end;
+	gint64 bound_end;
 	
 	bound_end = stream->bound_end != -1 ? stream->bound_end : null->written;
 	
@@ -190,7 +206,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 	return stream->position;
 }
 
-static off_t
+static gint64
 stream_tell (GMimeStream *stream)
 {
 	return stream->position;
@@ -199,8 +215,8 @@ stream_tell (GMimeStream *stream)
 static ssize_t
 stream_length (GMimeStream *stream)
 {
-	GMimeStreamNull *null = GMIME_STREAM_NULL (stream);
-	off_t bound_end;
+	GMimeStreamNull *null = (GMimeStreamNull *) stream;
+	gint64 bound_end;
 	
 	bound_end = stream->bound_end != -1 ? stream->bound_end : null->written;
 	
@@ -208,33 +224,31 @@ stream_length (GMimeStream *stream)
 }
 
 static GMimeStream *
-stream_substream (GMimeStream *stream, off_t start, off_t end)
+stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
-	GMimeStreamNull *null;
+	GMimeStream *null;
 	
-	null = g_object_new (GMIME_TYPE_STREAM_NULL, NULL, NULL);
+	null = g_object_newv (GMIME_TYPE_STREAM_NULL, 0, NULL);
+	g_mime_stream_construct (null, start, end);
 	
-	g_mime_stream_construct (GMIME_STREAM (null), start, end);
-	
-	return GMIME_STREAM (null);
+	return null;
 }
 
 
 /**
  * g_mime_stream_null_new:
  *
- * Creates a new GMimeStreamNull object.
+ * Creates a new #GMimeStreamNull object.
  *
- * Returns a new null stream (similar to /dev/null on Unix).
+ * Returns: a new null stream (similar to /dev/null on Unix).
  **/
 GMimeStream *
 g_mime_stream_null_new (void)
 {
-	GMimeStreamNull *null;
+	GMimeStream *null;
 	
-	null = g_object_new (GMIME_TYPE_STREAM_NULL, NULL, NULL);
+	null = g_object_newv (GMIME_TYPE_STREAM_NULL, 0, NULL);
+	g_mime_stream_construct (null, 0, -1);
 	
-	g_mime_stream_construct (GMIME_STREAM (null), 0, -1);
-	
-	return GMIME_STREAM (null);
+	return null;
 }
