@@ -213,7 +213,6 @@ static void
 test_multipart_signed (GMimeCipherContext *ctx)
 {
 	GMimeSignatureValidity *validity;
-	GMimeContentType *content_type;
 	GMimeMultipartSigned *mps;
 	GMimeDataWrapper *content;
 	GMimeMessage *message;
@@ -223,9 +222,7 @@ test_multipart_signed (GMimeCipherContext *ctx)
 	GMimePart *part;
 	Exception *ex;
 	
-	part = g_mime_part_new ();
-	content_type = g_mime_content_type_new ("text", "plain");
-	g_mime_object_set_content_type ((GMimeObject *) part, content_type);
+	part = g_mime_part_new_with_type ("text", "plain");
 	
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_write_string (stream, MULTIPART_SIGNED_CONTENT);
@@ -312,7 +309,6 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 {
 	const GMimeSignatureValidity *sv;
 	GMimeStream *cleartext, *stream;
-	GMimeContentType *content_type;
 	GMimeMultipartEncrypted *mpe;
 	GMimeDataWrapper *content;
 	GMimeObject *decrypted;
@@ -332,9 +328,7 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 	g_mime_data_wrapper_set_stream (content, cleartext);
 	g_object_unref (cleartext);
 	
-	part = g_mime_part_new ();
-	content_type = g_mime_content_type_new ("text", "plain");
-	g_mime_object_set_content_type ((GMimeObject *) part, content_type);
+	part = g_mime_part_new_with_type ("text", "plain");
 	g_mime_part_set_content_object (part, content);
 	g_object_unref (content);
 	
@@ -398,7 +392,6 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 	if (!decrypted || err != NULL) {
 		ex = exception_new ("decryption failed: %s", err->message);
 		g_object_unref (cleartext);
-		g_object_unref (mpe);
 		g_error_free (err);
 		throw (ex);
 	}
@@ -414,8 +407,6 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 			ex = exception_new ("signature validity status expected to be NONE");
 	}
 	
-	g_object_unref (mpe);
-	
 	if (ex != NULL) {
 		g_object_unref (cleartext);
 		throw (ex);
@@ -423,7 +414,6 @@ test_multipart_encrypted (GMimeCipherContext *ctx, gboolean sign)
 	
 	stream = g_mime_stream_mem_new ();
 	g_mime_object_write_to_stream (decrypted, stream);
-	g_object_unref (decrypted);
 	
 	buf[0] = GMIME_STREAM_MEM (cleartext)->buffer;
 	buf[1] = GMIME_STREAM_MEM (stream)->buffer;
